package me.hammerle.snuviengine.game;

import java.util.List;
import me.hammerle.snuviengine.api.Shader;
import me.hammerle.snuviengine.game.BoxList.Node;

public class Entity
{
    public Game game;
    
    public double xPos;
    public double yPos;
    public double width;
    public double height;
    
    public double vx = 0.0f;
    public double vy = 0.0f;
    
    public Node node;
    
    public int counter = 0;
    
    public Entity(Game game, double width, double height)
    {
        this.game = game;
        xPos = 0.0f;
        yPos = 0.0f;
        this.width = width;
        this.height = height;
        
        vx = Math.random() * 4 - 2;
        vy = Math.random() * 4 - 2;
    }
    
    private boolean isColliding(Entity ent)
    {
        return xPos + width > ent.xPos && ent.xPos + ent.width > xPos &&
                yPos + height > ent.yPos && ent.yPos + ent.height > yPos;
    }
    
    public boolean isColliding(double minX, double minY, double maxX, double maxY)
    {
        return xPos + width > minX && maxX > xPos &&
                yPos + height > minY && maxY > yPos;
    }
    
    public void tick()
    {
        counter++;
        if(counter == 200)
        {
            counter = 0;
            vx = Math.random() * 4 - 2;
            vy = Math.random() * 4 - 2;
        }
        
        if(vx == 0.0 && vy == 0.0)
        {
            return;
        }
        
        double step = 0.0625;
        double lvx = vx;
        double lvy = vy;
        
        game.entities.remove(node);
        node = null;
        
        List<Entity> list = game.entities.getAllEntitiesAt(this, 
                xPos + ((vx < 0) ? vx : 0), 
                yPos + ((vy < 0) ? vy : 0), 
                xPos + width + ((vx > 0) ? vx : 0), 
                yPos + height + ((vy > 0) ? vy : 0));
        if(list.isEmpty())
        {
            xPos += vx;
            if(xPos < 0 || xPos > 400)
            {
                xPos -= vx;
            }
            
            yPos += vy;
            if(yPos < 0 || yPos > 300)
            {
                yPos -= vy;
            }
            
            node = game.entities.add(this);
            return;
        }
        
        while(lvx != 0.0 || lvy != 0.0)
        {
            double oldXPos = xPos;
            double oldYPos = yPos;
            
            if(lvx < 0.0)
            {
                if(lvx > -step)
                {
                    xPos += lvx;
                    lvx = 0.0;
                }
                else
                {
                    xPos -= step;
                    lvx += step;
                }
            }
            else if(lvx > 0.0)
            {
                if(lvx < step)
                {
                    xPos += lvx;
                    lvx = 0.0;
                }
                else
                {
                    xPos += step;
                    lvx -= step;
                }
            }
            
            if(xPos > 400 || xPos < 0)
            {
                lvx = 0.0f;
                xPos = oldXPos;
            }
            else
            {
                for(Entity ent : list)
                {
                    if(this.isColliding(ent))
                    {
                        lvx = 0.0f;
                        xPos = oldXPos;
                        break;
                    }
                }
            }
            
            if(lvy < 0.0)
            {
                if(lvy > -step)
                {
                    yPos += lvy;
                    lvy = 0.0;
                }
                else
                {
                    yPos -= step;
                    lvy += step;
                }
            }
            else if(lvy > 0.0)
            {
                if(lvy < step)
                {
                    yPos += lvy;
                    lvy = 0.0;
                }
                else
                {
                    yPos += step;
                    lvy -= step;
                }
            }
            
            if(yPos > 300 || yPos < 0)
            {
                lvy = 0.0f;
                yPos = oldYPos;
            }
            else
            {
                for(Entity ent : list)
                {
                    if(this.isColliding(ent))
                    {
                        lvy = 0.0f;
                        yPos = oldYPos;
                        break;
                    }
                }
            }
        }
        
        node = game.entities.add(this);
    }
    
    public void renderTick(double lag)
    {
        Shader.getColorRenderer().setDepth((float) yPos);
        Shader.getColorRenderer().drawRectangle(
                (float) xPos, 
                (float) yPos, 
                (float) (xPos + width), 
                (float) (yPos + height), 
                0x770000FF);
    }
    
    public double getX()
    {
        return xPos;
    }
    
    public double getY()
    {
        return yPos;
    }
    
    public void setPosition(double x, double y)
    {
        xPos = x;
        yPos = y;
    }
}
